searchData={"items":[{"type":"module","title":"Buckets","doc":"Provides a generic interface for uploading files to buckets hosted by\ndifferent cloud providers.","ref":"Buckets.html"},{"type":"function","title":"Buckets.delete/2","doc":"Delegates a `delete/2` function call to the configured `:strategy`.","ref":"Buckets.html#delete/2"},{"type":"function","title":"Buckets.get/2","doc":"Delegates a `get/2` function call to the configured `:strategy`.","ref":"Buckets.html#get/2"},{"type":"function","title":"Buckets.put/3","doc":"Delegates a `put/3` function call to the configured `:strategy`.","ref":"Buckets.html#put/3"},{"type":"function","title":"Buckets.url/2","doc":"Delegates a `url/2` function call to the configured `:strategy`.","ref":"Buckets.html#url/2"},{"type":"behaviour","title":"Buckets.Cloud","doc":"Defines a cloud.\n\nA cloud manages the movement of files and data between your application and\nremote buckets for persistent storage.\n\nWhen used, it expects `:otp_app` and `:default_location` as options:\n\n    defmodule MyApp.Cloud do\n      use Buckets.Cloud,\n        otp_app: :my_app,\n        default_location: :local\n    end\n\nAdditional configuration is fetched from the application config, using a\ncombination of `:otp_app` and the module that you defined. You may configure\nmany \"locations\" in your config, for setting up multi-cloud support:\n\n      config :my_app, MyApp.Cloud,\n        locations: [\n          local: [\n            strategy: Buckets.Strategy.Volume,\n            # configure this strategy...\n          ],\n          gcs: [\n            strategy: Buckets.Strategy.GCS,\n            # configure this strategy...\n          ],\n          us_east_1: [\n            strategy: Buckets.Strategy.S3,\n            # configure this strategy...\n          ]\n        ]\n\nYou may also specify config dynamically at runtime, using the `:config` opt\nwhere it is supported.\n\nWhen building a multi-cloud application and persisting uploaded objects to a\ndatabase, you should take care to store some indicator of the location config that\nwas used to insert a particular object. Otherwise, you won't know how to fetch\ndata for or manage that object the next time it is accessed.\n\nSimilarly, if you are building a dynamic-cloud application (say, so that your users\ncan specify their own clouds for your application to use), you should store the\nconfiguration provided by the user. As objects are inserted to the cloud and stored\nin your database, add a mapping between the object and the user-provided config. If\nthis config is ever deleted, objects managed by it will become inaccessible. Keep this\nin mind as you build features to let users update or delete their config.","ref":"Buckets.Cloud.html"},{"type":"callback","title":"Buckets.Cloud.delete/1","doc":"Deletes a `Buckets.Object` permanently.","ref":"Buckets.Cloud.html#c:delete/1"},{"type":"callback","title":"Buckets.Cloud.delete!/1","doc":"Same as `c:delete/1` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:delete!/1"},{"type":"callback","title":"Buckets.Cloud.insert/2","doc":"Inserts a `Buckets.Object` or a file from a path into a bucket.","ref":"Buckets.Cloud.html#c:insert/2"},{"type":"callback","title":"Buckets.Cloud.insert!/2","doc":"Same as `c:insert/2` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:insert!/2"},{"type":"callback","title":"Buckets.Cloud.live_upload/2","doc":"Returns a map to be used as configuration for a LiveView live upload. The configuration\ncontains a signed URL that permits the upload to a remote bucket. Requires that an `:uploader`\nis configured for the location that the file is being uploaded to.","ref":"Buckets.Cloud.html#c:live_upload/2"},{"type":"callback","title":"Buckets.Cloud.live_upload!/2","doc":"Same as `c:live_upload/1` but returns the upload config or raises if there is an error.","ref":"Buckets.Cloud.html#c:live_upload!/2"},{"type":"callback","title":"Buckets.Cloud.load/2","doc":"Loads the data for a `Buckets.Object`, placing it in memory by default. It will load data\nlazily, doing nothing if data is already present.","ref":"Buckets.Cloud.html#c:load/2"},{"type":"callback","title":"Options - Buckets.Cloud.load/2","doc":"* `:to` - A location to store the loaded data to on disk, if this is preferred over\n      loading to memory. May be a path, `:tmp` (to load to the configured tmp dir), or\n      `{:tmp, path}` (to load to a path in the configured tmp dir).\n    * `:force` - If data is already present, first unload it with `c:unload/1` before loading\n      new data. Warning: if data is stored in a file, it will be deleted.","ref":"Buckets.Cloud.html#c:load/2-options"},{"type":"callback","title":"Buckets.Cloud.load!/2","doc":"Same as `c:load/1` but returns the object or raises if there is an error.","ref":"Buckets.Cloud.html#c:load!/2"},{"type":"callback","title":"Buckets.Cloud.normalize_filename/1","doc":"An overridable function that processes filenames before encoding them in a remote path.\n\nBy default, replaces whitespace with `\"_\"` and removes all non-alphanumeric characters.","ref":"Buckets.Cloud.html#c:normalize_filename/1"},{"type":"callback","title":"Buckets.Cloud.read/1","doc":"Reads the data for `Buckets.Object`, preferring local data first and fetching from\nthe remote bucket if needed.","ref":"Buckets.Cloud.html#c:read/1"},{"type":"callback","title":"Buckets.Cloud.read!/1","doc":"Same as `c:read/1` but returns the data or raises if there is an error.","ref":"Buckets.Cloud.html#c:read!/1"},{"type":"callback","title":"Buckets.Cloud.tmp_dir/0","doc":"An overridable function that specifies the temporary directory in which objects are stored.\n\nDefaults to `System.tmp_dir!()`.","ref":"Buckets.Cloud.html#c:tmp_dir/0"},{"type":"callback","title":"Buckets.Cloud.unload/1","doc":"Unloads the data for a `Buckets.Object`. If the data is stored in a local file, the file will\nbe deleted.","ref":"Buckets.Cloud.html#c:unload/1"},{"type":"callback","title":"Buckets.Cloud.url/1","doc":"Returns a SignedURL struct for a `Buckets.Object`.","ref":"Buckets.Cloud.html#c:url/1"},{"type":"callback","title":"Buckets.Cloud.url!/2","doc":"Same as `c:url/1` but returns the SignedURL raises if there is an error.","ref":"Buckets.Cloud.html#c:url!/2"},{"type":"module","title":"Buckets.Cloud.Operations","doc":"","ref":"Buckets.Cloud.Operations.html"},{"type":"function","title":"Buckets.Cloud.Operations.delete/1","doc":"","ref":"Buckets.Cloud.Operations.html#delete/1"},{"type":"function","title":"Buckets.Cloud.Operations.delete!/1","doc":"","ref":"Buckets.Cloud.Operations.html#delete!/1"},{"type":"function","title":"Buckets.Cloud.Operations.insert/3","doc":"","ref":"Buckets.Cloud.Operations.html#insert/3"},{"type":"function","title":"Buckets.Cloud.Operations.insert!/3","doc":"","ref":"Buckets.Cloud.Operations.html#insert!/3"},{"type":"function","title":"Buckets.Cloud.Operations.live_upload/3","doc":"","ref":"Buckets.Cloud.Operations.html#live_upload/3"},{"type":"function","title":"Buckets.Cloud.Operations.live_upload!/3","doc":"","ref":"Buckets.Cloud.Operations.html#live_upload!/3"},{"type":"function","title":"Buckets.Cloud.Operations.load/3","doc":"","ref":"Buckets.Cloud.Operations.html#load/3"},{"type":"function","title":"Buckets.Cloud.Operations.load!/3","doc":"","ref":"Buckets.Cloud.Operations.html#load!/3"},{"type":"function","title":"Buckets.Cloud.Operations.read/1","doc":"","ref":"Buckets.Cloud.Operations.html#read/1"},{"type":"function","title":"Buckets.Cloud.Operations.read!/1","doc":"","ref":"Buckets.Cloud.Operations.html#read!/1"},{"type":"function","title":"Buckets.Cloud.Operations.unload/1","doc":"","ref":"Buckets.Cloud.Operations.html#unload/1"},{"type":"function","title":"Buckets.Cloud.Operations.url/2","doc":"","ref":"Buckets.Cloud.Operations.html#url/2"},{"type":"function","title":"Buckets.Cloud.Operations.url!/2","doc":"","ref":"Buckets.Cloud.Operations.html#url!/2"},{"type":"module","title":"Buckets.Location","doc":"","ref":"Buckets.Location.html"},{"type":"function","title":"Buckets.Location.new/2","doc":"","ref":"Buckets.Location.html#new/2"},{"type":"type","title":"Buckets.Location.t/0","doc":"","ref":"Buckets.Location.html#t:t/0"},{"type":"module","title":"Buckets.Location.NotConfigured","doc":"","ref":"Buckets.Location.NotConfigured.html"},{"type":"module","title":"Buckets.Object","doc":"","ref":"Buckets.Object.html"},{"type":"function","title":"Buckets.Object.from_file/1","doc":"","ref":"Buckets.Object.html#from_file/1"},{"type":"function","title":"Buckets.Object.from_upload/1","doc":"","ref":"Buckets.Object.html#from_upload/1"},{"type":"function","title":"Buckets.Object.new/3","doc":"","ref":"Buckets.Object.html#new/3"},{"type":"function","title":"Buckets.Object.read/1","doc":"","ref":"Buckets.Object.html#read/1"},{"type":"function","title":"Buckets.Object.read!/1","doc":"","ref":"Buckets.Object.html#read!/1"},{"type":"type","title":"Buckets.Object.t/0","doc":"","ref":"Buckets.Object.html#t:t/0"},{"type":"module","title":"Buckets.Router","doc":"","ref":"Buckets.Router.html"},{"type":"macro","title":"Buckets.Router.buckets_volume/2","doc":"","ref":"Buckets.Router.html#buckets_volume/2"},{"type":"function","title":"Buckets.Router.scope/0","doc":"","ref":"Buckets.Router.html#scope/0"},{"type":"module","title":"Buckets.Router.VolumeController","doc":"","ref":"Buckets.Router.VolumeController.html"},{"type":"function","title":"Buckets.Router.VolumeController.get/2","doc":"","ref":"Buckets.Router.VolumeController.html#get/2"},{"type":"function","title":"Buckets.Router.VolumeController.put/2","doc":"","ref":"Buckets.Router.VolumeController.html#put/2"},{"type":"module","title":"Buckets.SignedURL","doc":"","ref":"Buckets.SignedURL.html"},{"type":"function","title":"Buckets.SignedURL.to_string/1","doc":"","ref":"Buckets.SignedURL.html#to_string/1"},{"type":"type","title":"Buckets.SignedURL.t/0","doc":"","ref":"Buckets.SignedURL.html#t:t/0"},{"type":"behaviour","title":"Buckets.Strategy","doc":"","ref":"Buckets.Strategy.html"},{"type":"callback","title":"Buckets.Strategy.delete/2","doc":"","ref":"Buckets.Strategy.html#c:delete/2"},{"type":"callback","title":"Buckets.Strategy.get/2","doc":"","ref":"Buckets.Strategy.html#c:get/2"},{"type":"callback","title":"Buckets.Strategy.put/3","doc":"","ref":"Buckets.Strategy.html#c:put/3"},{"type":"callback","title":"Buckets.Strategy.url/2","doc":"","ref":"Buckets.Strategy.html#c:url/2"},{"type":"module","title":"Buckets.Strategy.GCS","doc":"","ref":"Buckets.Strategy.GCS.html"},{"type":"function","title":"Buckets.Strategy.GCS.url/2","doc":"Gets a signed URL for temporarily delegating access to an object in a bucket.","ref":"Buckets.Strategy.GCS.html#url/2"},{"type":"function","title":"Errors - Buckets.Strategy.GCS.url/2","doc":"* 403 PERMISSION_DENIED: Permission 'iam.serviceAccounts.signBlob' denied on resource\n      (or it may not exist). Make sure the authorized SA has role roles/iam.serviceAccountTokenCreator\n      on the SA passed in the URL.\n\n    * 403 PERMISSION_DENIED: IAM Service Account Credentials API has not been used in project {project_id}\n      before or it is disabled. Enable it by visiting {url} then retry. If you enabled this API recently,\n      wait a few minutes for the action to propagate to our systems and retry. Make sure the authorized SA\n      has role roles/iam.serviceAccountTokenCreator on the SA passed in the URL.\"","ref":"Buckets.Strategy.GCS.html#url/2-errors"},{"type":"module","title":"Buckets.Strategy.S3","doc":"","ref":"Buckets.Strategy.S3.html"},{"type":"module","title":"Buckets.Strategy.Volume","doc":"","ref":"Buckets.Strategy.Volume.html"},{"type":"function","title":"Buckets.Strategy.Volume.verify_signed_path/3","doc":"","ref":"Buckets.Strategy.Volume.html#verify_signed_path/3"},{"type":"module","title":"Buckets.Util","doc":"","ref":"Buckets.Util.html"},{"type":"function","title":"Buckets.Util.normalize_filename/1","doc":"","ref":"Buckets.Util.html#normalize_filename/1"},{"type":"function","title":"Buckets.Util.size/1","doc":"","ref":"Buckets.Util.html#size/1"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}